/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/


/***********************************************************************
MultiCam C++ Sample program.

Acquires an image from the first board and save it in a ppm file.
The board must come from Picolo family.

The ppm file can be easily viewed with most image display programs,
like gimp, kview, xv.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "/usr/local/euresys/multicam/include/MultiCamCpp.h"

#ifndef BYTE
#define BYTE unsigned char
#endif

using namespace Euresys::MultiCam;

void ProcessImage(BYTE *pImage, int sizeX, int sizeY, int *saved_images);
void SaveImage(BYTE *pImage, int sizeX, int sizeY, int *saved_images);

class CPicoloGrab
{
public:
    unsigned int imageBuffer;
    int sizex, sizey, bufferPitch;
    int saved_images;
    Channel *pChannel;

    CPicoloGrab();
    ~CPicoloGrab();
    BOOL Init();
    void AcquireImages();
    void ProcessingCallback(Channel &channel, SignalInfo &info);
    void AcqFailureCallback(Channel &channel, SignalInfo &info);
};


CPicoloGrab::CPicoloGrab()
{
    /* Initialize the MultiCam driver.
    Do it once at the beginning of your program.
    */
    sizex = 0;
    sizey = 0;
    saved_images = 0;
    pChannel = NULL;
}

CPicoloGrab::~CPicoloGrab()
{
    if (pChannel != NULL)
        delete pChannel;
}

BOOL CPicoloGrab::Init()
{
    try
    {
        // Create a channel and associate it with the first connector on the first board
        pChannel = new Channel(Boards[0], "VID1");
        // Choose the video standard
        pChannel->SetParam(MC_CamFile, "PAL");
        // Choose the pixel color format
        pChannel->SetParam(MC_ColorFormat, MC_ColorFormat_RGB24);
        // Choose the acquisition mode
        pChannel->SetParam(MC_AcquisitionMode, MC_AcquisitionMode_VIDEO);
        // Choose the way the first acquisition is triggered
        pChannel->SetParam(MC_TrigMode, MC_TrigMode_IMMEDIATE);
        // Choose the triggering mode for subsequent acquisitions
        pChannel->SetParam(MC_NextTrigMode, MC_NextTrigMode_REPEAT);
        // Choose the number of images to acquire
        pChannel->SetParam(MC_SeqLength_Fr, 1);

        /* Here is the ideal place to adjust some other acquisition parameters,
        like image size,grab window, gain or offset.*/

        /* Get the image width and height. 
        Will be used to allocate the right amount of memory 
        and for processing the image
        */
        pChannel->GetParam(MC_ImageSizeX, sizex);
        pChannel->GetParam(MC_ImageSizeY, sizey);

        /* The number of images to acquire.
        MC_INFINITE: infinite
        */
        pChannel->SetParam(MC_SeqLength_Fr, 1);

        // Register the callback function called when a new image is available
        pChannel->RegisterCallback(this, &CPicoloGrab::ProcessingCallback, MC_SIG_SURFACE_PROCESSING);
        pChannel->RegisterCallback(this, &CPicoloGrab::AcqFailureCallback, MC_SIG_ACQUISITION_FAILURE);

        /* Enable the signals we need: 
        MC_SIG_SURFACE_PROCESSING: acquisition done and locked for processing
        MC_SIG_ACQUISITION_FAILURE: acquisition failed.
        */
        // Enable the signal corresponding to the callback function 
        pChannel->SetParam(MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
        pChannel->SetParam(MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, MC_SignalEnable_ON);

        // Prepare the channel in order to minimize the acquisition sequence startup latency
        pChannel->Prepare();
    }
    catch(Euresys::MultiCam::Exception &e)
    {
        printf("Error : %s\n", e.What());

        if (pChannel != NULL)
            delete pChannel;

        return FALSE;
    }

    return TRUE;
}

/*
Acquire images and process them.
*/
void CPicoloGrab::AcquireImages()
{
    /*
    Start Acquisitions for this channel.
    Acquisition will start when the channel is active.
    */
    fprintf(stderr,"SetParam MC_ChannelState <MC_ChannelState_ACTIVE>\n");
    pChannel->SetParam(MC_ChannelState, MC_ChannelState_ACTIVE);

    fprintf(stderr,"WAITING for image processing callbacks\n");

    /* Wait 3 seconds. That should be enough for the acquisition to finish.
    In industrial programs, this will probably be replaced with some wait
    for the end of the application.
    */
    sleep(3);

    fprintf(stderr,"SetParam MC_ChannelState <MC_ChannelState_IDLE>\n");
    pChannel->SetParam(MC_ChannelState, MC_ChannelState_IDLE);
}

/*
Callback function registered to our channel. 
Called at the asynchronous signals.
*/
void CPicoloGrab::ProcessingCallback(Channel &Ch, SignalInfo &Info)
{
    try
    {
        fprintf(stderr,"Processing surface.\n");

        // Update the bitmap image with the acquired image data 
        pChannel->GetParam(MC_ImageSizeX, sizex);
        pChannel->GetParam(MC_ImageSizeY, sizey);
        Info.Surf->GetParam(MC_SurfacePitch, bufferPitch);
        Info.Surf->GetParam(MC_SurfaceAddr, imageBuffer);

        ProcessImage(reinterpret_cast<unsigned char*>(imageBuffer), sizex, sizey, &saved_images);

        /* Insert image analysis and processing code here */

    }
    catch (Euresys::MultiCam::Exception &e)
    {
        printf("Error during callback processing : %s\n", e.What());
    }
}

void CPicoloGrab::AcqFailureCallback(Channel &channel, SignalInfo &info)
{
    fprintf(stderr, "Acquisition Failure. Is a video signal connected ?\n");
}

void ProcessImage(BYTE *pImage, int sizeX, int sizeY, int *saved_images)
{
    SaveImage(pImage, sizeX, sizeY, saved_images);
}

/*
Save the image in a .ppm file.
*/
void SaveImage(BYTE *pImage, int sizeX, int sizeY, int *saved_images)
{
    FILE *f;
    char filename[256];
    int i;

    sprintf(filename,"img_%04d.ppm", *saved_images++);
    f=fopen(filename,"w");

    fprintf(f,"P6\n%d\n%d\n255\n",sizeX,sizeY);
    for(i = 0; i < sizeX * sizeY * 3; i += 3)
    {
        fputc(pImage[i+2],f);
        fputc(pImage[i+1],f);
        fputc(pImage[i+0],f);
    }

    fclose(f);
}

int main(int argc, char* argv[])
{
    CPicoloGrab *PicoloGrab;

    try
    {
        /* Initialize the MultiCam driver. */
        Euresys::MultiCam::Initialize();

        PicoloGrab = new CPicoloGrab();

        if (PicoloGrab->Init() == FALSE)
        {
            delete PicoloGrab;

            /* Close the MultiCam driver. */
            Euresys::MultiCam::Terminate();

            return -1;
        }

        /* Acquire the images and process them.*/
        PicoloGrab->AcquireImages();

        delete PicoloGrab;

        /* Close the MultiCam driver. */
        Euresys::MultiCam::Terminate();
    }
    catch(Euresys::MultiCam::Exception &e)
    {
        printf("Error during image grabbing : %s\n", e.What());
    }


    return 0;
}
